function [x,lam,xi] = cgls(m,x0,d,epsr,nits,tol)

% conjugate gradient iteration for least squares problem
%     min ||m*x - d||
% also returns estimates of the singular values (lam) and
% right singular vectors (xi) of the matrix a on the Krylov
% subspace.
%
% cast:
%
% x            solution estimate (least-norm 
%              solution as approximated by CG)
% lam          vector of Lanczos singular values (nits x 1)
% xi           matrix of Lanczos singular vectors (nits x nits)
% m            matrix 
% x0           initial estimate;
% d            data vector
% epsr         residual tolerance
% nits         maximum iterations
% tol          anti zerodivide tolerance

% initialization

fprintf('\nCG by MATLAB\n');

% lanczos tridiagonal matrix
lanc =zeros(nits,nits);

% normal matrix
a=m'*m;

% rhs in normal equations
b=m'*d;

% initial residual
r=d-m*x0;

% initial residual norm
rnorm = sqrt(r'*r);

% initial normal residual
s=m'*r;

% initialize solution
x=x0;

% main loop

for k = 1:nits
  if k == 1
     beta = 0.0;
     p = s;
     stsc = s'*s;
  else 
     if stsp < tol
        fprintf(' Error: CG, stsp too small\n');
        break
     end
     beta = stsc/stsp;
     p = s + beta*p;
  end
  
  mp = m*p;
  ap = m'*mp;
  ptap = p'*ap;
  if ptap < tol
     fprintf(' Error: CG, ptap too small\n');
     break
  end

% update solution

  alphac = stsc/ptap;
  x = x + alphac*p;

% update Lanczos matrix

  if k==1
     lanc(1,1)=1.0/alphac;
  else
     lanc(k-1,k)=-sqrt(stsc/stsp)/alphap;
     lanc(k,k-1)=lanc(k-1,k);
     lanc(k,k)  =stsc/(stsp*alphap) + 1.0/alphac;
  end

% find eigenvalues

  [xi,evl]=eig(lanc(1:k,1:k));
  lam=diag(evl);

% update residuals

  r = r - alphac*mp;
  s = s - alphac*ap;

% update scalars

  stsp   = stsc;
  stsc   = s'*s;
  rnorm  = sqrt(r'*r);
  alphap = alphac;

% announce results

  fprintf('step %g residual %g\n',k,stsc);
%  fprintf('Lanczos estimates of eigenvalues:\n') 
%  for j=1:k
%    fprintf('index %g eigenvalue %g\n',j,lam(j));
%  end

% test for convergence

  if sqrt(rnorm) < epsr
     fprintf('\nConvergence: res = %g\n',rnorm)
     break;
  end

end


  
